ant_config_example = {
    "antennaPort": 1,
    "estimatedTagPopulation": 16,
    "fastId": "enabled",
    "inventorySearchMode": "dual-target",
    "inventorySession": 1,
    "receiveSensitivityDbm": -80,
    "rfMode": 4,
    "transmitPowerCdbm": 3300,
    "protectedModePinHex": "12345678",
}

R700_IOT_config_example = {
    "antennaConfigs": [
        {
            "antennaPort": 1,
            "estimatedTagPopulation": 16,
            "fastId": "enabled",
            "inventorySearchMode": "dual-target",
            "inventorySession": 1,
            "receiveSensitivityDbm": -80,
            "rfMode": 4,
            "transmitPowerCdbm": 3300,
        },
        {
            "antennaPort": 2,
            "estimatedTagPopulation": 16,
            "fastId": "enabled",
            "inventorySearchMode": "dual-target",
            "inventorySession": 1,
            "receiveSensitivityDbm": -80,
            "rfMode": 4,
            "transmitPowerCdbm": 3300,
        },
        {
            "antennaPort": 3,
            "estimatedTagPopulation": 16,
            "fastId": "enabled",
            "inventorySearchMode": "dual-target",
            "inventorySession": 1,
            "receiveSensitivityDbm": -80,
            "rfMode": 4,
            "transmitPowerCdbm": 3300,
        },
        {
            "antennaPort": 4,
            "estimatedTagPopulation": 16,
            "fastId": "enabled",
            "inventorySearchMode": "dual-target",
            "inventorySession": 1,
            "receiveSensitivityDbm": -80,
            "rfMode": 4,
            "transmitPowerCdbm": 3300,
        },
    ],
    "startTriggers": [{"gpiTransitionEvent": {"gpi": 1, "transition": "high-to-low"}}],
    "stopTriggers": [{"gpiTransitionEvent": {"gpi": 1, "transition": "low-to-high"}}],
    "eventConfig": {
        "common": {"hostname": "disabled"},
        "tagInventory": {
            "epc": "disabled",
            "epcHex": "enabled",
            "xpcHex": "disabled",
            "tid": "disabled",
            "tidHex": "enabled",
            "antennaPort": "enabled",
            "transmitPowerCdbm": "disabled",
            "peakRssiCdbm": "enabled",
            "frequency": "disabled",
            "pc": "disabled",
            "lastSeenTime": "disabled",
            "phaseAngle": "disabled",
            "tagReporting": {
                "reportingIntervalSeconds": 1,
                "tagCacheSize": 2048,
                "antennaIdentifier": "antennaPort",
                "tagIdentifier": "epc",
            },
        },
    },
}
